#!/usr/bin/env python

'''
Copyright 2023 PySimpleSoft, Inc. and/or its licensors. All rights reserved.

Redistribution, modification, or any other use of PySimpleGUI or any portion thereof is subject
to the terms of the PySimpleGUI License Agreement available at https://eula.pysimplegui.com.

You may not redistribute, modify or otherwise use PySimpleGUI or its contents except pursuant
to the PySimpleGUI License Agreement.
'''

import PySimpleGUI as sg
import sys
import psutil

"""
    Desktop floating widget - CPU Cores 
    Right click the window to exit
    Uses psutil to display:
        CPU usage of each individual core
    CPU utilization is updated every 500 ms by default
    Utiliziation is shown as a scrolling area graph
    To achieve a "rainmeter-style" of window, these featurees were used:
      An alpha-channel setting of 0.8 to give a little transparency
      No titlebar
      Grab anywhere, making window easy to move around
    Note that the keys are tuples, with a tuple as the second item
        ('-KEY-', (row, col))      
"""

GRAPH_WIDTH = 120       # each individual graph size in pixels
GRAPH_HEIGHT = 40
TRANSPARENCY = .8       # how transparent the window looks. 0 = invisible, 1 = normal window
NUM_COLS = 4
POLL_FREQUENCY = 2500    # how often to update graphs in milliseconds

colors = ('#23a0a0', '#56d856', '#be45be', '#5681d8', '#d34545', '#BE7C29')

# DashGraph does the drawing of each graph
class DashGraph(object):
    def __init__(self, graph_elem, text_elem, starting_count, color):
        self.graph_current_item = 0
        self.graph_elem = graph_elem        # type: sg.Graph
        self.text_elem = text_elem
        self.prev_value = starting_count
        self.max_sent = 1
        self.color = color
        self.line_list = []                 # list of currently visible lines. Used to delete oild figures

    def graph_percentage_abs(self, value):
        self.line_list.append(self.graph_elem.draw_line(            # draw a line and add to list of lines
            (self.graph_current_item, 0),
            (self.graph_current_item, value),
            color=self.color))
        if self.graph_current_item >= GRAPH_WIDTH:
            self.graph_elem.move(-1,0)
            self.graph_elem.delete_figure(self.line_list[0])        # delete the oldest line
            self.line_list = self.line_list[1:]                     # remove line id from list of lines
        else:
            self.graph_current_item += 1

    def text_display(self, text):
        self.text_elem.update(text)

def main():
    # A couple of "User defined elements" that combine several elements and enable bulk edits
    def Txt(text, **kwargs):
        return(sg.Text(text, font=('Helvetica 8'), **kwargs))

    def GraphColumn(name, key):
        return sg.Column([[Txt(name, size=(10,1), key=('-TXT-', key))],
                          [sg.Graph((GRAPH_WIDTH, GRAPH_HEIGHT), (0, 0), (GRAPH_WIDTH, 100), background_color='black', key=('-GRAPH-', key))]], pad=(2, 2))

    num_cores = len(psutil.cpu_percent(percpu=True))        # get the number of cores in the CPU

    sg.theme('Black')

    layout = [[sg.Text('CPU Core Usage', justification='c', expand_x=True)] ]

    # add on the graphs
    for rows in range(num_cores//NUM_COLS+1):
        # for cols in range(min(num_cores-rows*NUM_COLS, NUM_COLS)):
        layout += [[GraphColumn('CPU '+str(rows*NUM_COLS+cols), (rows, cols)) for cols in range(min(num_cores-rows*NUM_COLS, NUM_COLS))]]

    # ----------------  Create Window  ----------------
    window = sg.Window('CPU Cores Usage Widget', layout,
                       keep_on_top=True,
                       grab_anywhere=True,
                       no_titlebar=True,
                       return_keyboard_events=True,
                       alpha_channel=TRANSPARENCY,
                       use_default_focus=False,
                       finalize=True,
                       margins=(1,1),
                       element_padding=(0,0),
                       border_depth=0,
                       enable_close_attempted_event=True,
                       auto_save_location=True,
                       right_click_menu=sg.MENU_RIGHT_CLICK_EDITME_VER_EXIT)


    graphs = []
    for rows in range(num_cores//NUM_COLS+1):
        for cols in range(min(num_cores-rows*NUM_COLS, NUM_COLS)):
            graphs += [DashGraph(window[('-GRAPH-', (rows, cols))],
                                 window[('-TXT-',  (rows, cols))],
                                 0, colors[(rows*NUM_COLS+cols)%len(colors)])]


    # ----------------  main loop  ----------------
    while True :
        # --------- Read and update window once every Polling Frequency --------
        event, values = window.read(timeout=POLL_FREQUENCY)
        if event in (sg.WIN_CLOSE_ATTEMPTED_EVENT, 'Exit'):         # Be nice and give an exit
            break
        elif event == sg.WIN_CLOSED:
            break
        elif event == 'Edit Me':
            sg.execute_editor(__file__)
        elif event == 'Version':
            sg.popup_scrolled(__file__, sg.get_versions(), keep_on_top=True, location=window.current_location())
        # read CPU for each core
        stats = psutil.cpu_percent(percpu=True)

        sg.timer_start()
        # update each graph
        for i, util in enumerate(stats):
            graphs[i].graph_percentage_abs(util)
            graphs[i].text_display('{} CPU {:2.0f}'.format(i, util))
        time_elapsed = sg.timer_stop()
    window.close()

if __name__ == "__main__":
    main()


#!/usr/bin/env python

'''
Copyright 2023 PySimpleSoft, Inc. and/or its licensors. All rights reserved.

Redistribution, modification, or any other use of PySimpleGUI or any portion thereof is subject
to the terms of the PySimpleGUI License Agreement available at https://eula.pysimplegui.com.

You may not redistribute, modify or otherwise use PySimpleGUI or its contents except pursuant
to the PySimpleGUI License Agreement.
'''

import PySimpleGUI as sg
import psutil

"""
    Desktop floating widget - System status dashboard
    Uses psutil to display:
        Network I/O
        Disk I/O
        CPU Used
        Mem Used
    Information is updated once a second and is shown as an area graph that scrolls
    This is an unusual demo. It's using a class to implement the individual graphs. Could be a great
    design pattern in many ways.  See what you can do with it!
"""

GRAPH_WIDTH, GRAPH_HEIGHT = 120, 40       # each individual graph size in pixels
ALPHA = .7


class DashGraph(object):
    def __init__(self, graph_elem, starting_count, color):
        self.graph_current_item = 0
        self.graph_elem = graph_elem            # type:sg.Graph
        self.prev_value = starting_count
        self.max_sent = 1
        self.color = color
        self.graph_lines = []

    def graph_value(self, current_value):
        delta = current_value - self.prev_value
        self.prev_value = current_value
        self.max_sent = max(self.max_sent, delta)
        percent_sent = 100 * delta / self.max_sent
        line_id = self.graph_elem.draw_line((self.graph_current_item, 0), (self.graph_current_item, percent_sent), color=self.color)
        self.graph_lines.append(line_id)
        if self.graph_current_item >= GRAPH_WIDTH:
            self.graph_elem.delete_figure(self.graph_lines.pop(0))
            self.graph_elem.move(-1, 0)
        else:
            self.graph_current_item += 1
        return delta

    def graph_percentage_abs(self, value):
        self.graph_elem.draw_line((self.graph_current_item, 0), (self.graph_current_item, value), color=self.color)
        if self.graph_current_item >= GRAPH_WIDTH:
            self.graph_elem.move(-1, 0)
        else:
            self.graph_current_item += 1


def human_size(bytes, units=(' bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB')):
    """ Returns a human readable string reprentation of bytes"""
    return str(bytes) + units[0] if bytes < 1024 else human_size(bytes >> 10, units[1:])


def main():
    # ----------------  Create Window  ----------------
    sg.theme('Black')
    sg.set_options(element_padding=(0, 0), margins=(1, 1), border_width=0)

    def GraphColumn(name, key):
        layout = [
            [sg.Text(name, size=(18,1), font=('Helvetica 8'), key=key+'TXT_')],
            [sg.Graph((GRAPH_WIDTH, GRAPH_HEIGHT),
                      (0, 0),
                      (GRAPH_WIDTH, 100),
                      background_color='black',
                      key=key+'GRAPH_')]]
        return sg.Col(layout, pad=(2, 2))

    red_x = b"R0lGODlhEAAQAPeQAIsAAI0AAI4AAI8AAJIAAJUAAJQCApkAAJoAAJ4AAJkJCaAAAKYAAKcAAKcCAKcDA6cGAKgAAKsAAKsCAKwAAK0AAK8AAK4CAK8DAqUJAKULAKwLALAAALEAALIAALMAALMDALQAALUAALYAALcEALoAALsAALsCALwAAL8AALkJAL4NAL8NAKoTAKwbAbEQALMVAL0QAL0RAKsREaodHbkQELMsALg2ALk3ALs+ALE2FbgpKbA1Nbc1Nb44N8AAAMIWAMsvAMUgDMcxAKVABb9NBbVJErFYEq1iMrtoMr5kP8BKAMFLAMxKANBBANFCANJFANFEB9JKAMFcANFZANZcANpfAMJUEMZVEc5hAM5pAMluBdRsANR8AM9YOrdERMpIQs1UVMR5WNt8X8VgYMdlZcxtYtx4YNF/btp9eraNf9qXXNCCZsyLeNSLd8SSecySf82kd9qqc9uBgdyBgd+EhN6JgtSIiNuJieGHhOGLg+GKhOKamty1ste4sNO+ueenp+inp+HHrebGrefKuOPTzejWzera1O7b1vLb2/bl4vTu7fbw7ffx7vnz8f///wAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAACH5BAEAAJAALAAAAAAQABAAAAjUACEJHEiwYEEABniQKfNFgQCDkATQwAMokEU+PQgUFDAjjR09e/LUmUNnh8aBCcCgUeRmzBkzie6EeQBAoAAMXuA8ciRGCaJHfXzUMCAQgYooWN48anTokR8dQk4sELggBhQrU9Q8evSHiJQgLCIIfMDCSZUjhbYuQkLFCRAMAiOQGGLE0CNBcZYmaRIDLqQFGF60eTRoSxc5jwjhACFWIAgMLtgUocJFy5orL0IQRHAiQgsbRZYswbEhBIiCCH6EiJAhAwQMKU5DjHCi9gnZEHMTDAgAOw=="
    layout = [
        [sg.Text('System Status Dashboard'+' '*18),
         sg.Button('', image_data=red_x, button_color=('black', 'black'), key='Exit', tooltip='Closes window')],
        [GraphColumn('Net Out', '_NET_OUT_'),
         GraphColumn('Net In', '_NET_IN_')],
        [GraphColumn('Disk Read', '_DISK_READ_'),
         GraphColumn('Disk Write', '_DISK_WRITE_')],
        [GraphColumn('CPU Usage', '_CPU_'),
         GraphColumn('Memory Usage', '_MEM_')], ]

    window = sg.Window('PSG System Dashboard', layout,
             keep_on_top=True,
             grab_anywhere=True, no_titlebar=True,
             return_keyboard_events=True, alpha_channel=ALPHA,
             use_default_focus=False, finalize=True, auto_save_location=True, right_click_menu=sg.MENU_RIGHT_CLICK_EDITME_VER_EXIT)

    # setup graphs & initial values
    netio = psutil.net_io_counters()
    net_in = window['_NET_IN_GRAPH_']
    net_graph_in = DashGraph(net_in, netio.bytes_recv, '#23a0a0')
    net_out = window['_NET_OUT_GRAPH_']
    net_graph_out = DashGraph(net_out, netio.bytes_sent, '#56d856')

    diskio = psutil.disk_io_counters()
    disk_graph_write = DashGraph(window['_DISK_WRITE_GRAPH_'], diskio.write_bytes, '#be45be')
    disk_graph_read = DashGraph(window['_DISK_READ_GRAPH_'], diskio.read_bytes, '#5681d8')

    cpu_usage_graph = DashGraph(window['_CPU_GRAPH_'], 0, '#d34545')
    mem_usage_graph = DashGraph(window['_MEM_GRAPH_'], 0, '#BE7C29')

    # print(psutil.cpu_percent(percpu=True))
    # ----------------  main loop  ----------------
    while True :
        # --------- Read and update window once a second--------
        event, values = window.read(timeout=1000)
        # Be nice and give an exit, expecially since there is no titlebar
        if event in (sg.WIN_CLOSED, 'Exit'):
            break
        if event == 'Edit Me':
            sg.execute_editor(__file__)
        elif event == 'Version':
            sg.popup_scrolled(__file__, sg.get_versions(), location=window.current_location(), keep_on_top=True, non_blocking=True)
        # ----- Network Graphs -----
        netio = psutil.net_io_counters()
        write_bytes = net_graph_out.graph_value(netio.bytes_sent)
        read_bytes = net_graph_in.graph_value(netio.bytes_recv)
        window['_NET_OUT_TXT_'].update('Net out {}'.format(human_size(write_bytes)))
        window['_NET_IN_TXT_'].update('Net In {}'.format(human_size(read_bytes)))
        # ----- Disk Graphs -----
        diskio = psutil.disk_io_counters()
        write_bytes = disk_graph_write.graph_value(diskio.write_bytes)
        read_bytes = disk_graph_read.graph_value(diskio.read_bytes)
        window['_DISK_WRITE_TXT_'].update('Disk Write {}'.format(human_size(write_bytes)))
        window['_DISK_READ_TXT_'].update('Disk Read {}'.format(human_size(read_bytes)))
        # ----- CPU Graph -----
        cpu = psutil.cpu_percent(0)
        cpu_usage_graph.graph_percentage_abs(cpu)
        window['_CPU_TXT_'].update('{0:2.0f}% CPU Used'.format(cpu))
        # ----- Memory Graph -----
        mem_used = psutil.virtual_memory().percent
        mem_usage_graph.graph_percentage_abs(mem_used)
        window['_MEM_TXT_'].update('{}% Memory Used'.format(mem_used))


if __name__ == "__main__":
    main()



#!/usr/bin/env python

'''
Copyright 2023 PySimpleSoft, Inc. and/or its licensors. All rights reserved.

Redistribution, modification, or any other use of PySimpleGUI or any portion thereof is subject
to the terms of the PySimpleGUI License Agreement available at https://eula.pysimplegui.com.

You may not redistribute, modify or otherwise use PySimpleGUI or its contents except pursuant
to the PySimpleGUI License Agreement.
'''

import PySimpleGUI as sg
import psutil
import operator

"""
    PSUTIL "Top" CPU Processes - Desktop Widget
    
    Creates a floating CPU utilization window running something similar to a "top" command.

    Use the spinner to adjust the number of seconds between readings of the CPU utilizaiton
    Rather than calling the threading module this program uses the PySimpleGUI perform_long_operation method.
        The result is similar.  The function is run as a thread... the call is simply wrapped.
            
"""

# global used to communicate with thread.
g_interval = 6      # how often to poll for CPU usage

def CPU_thread(window:sg.Window):
    # window.write_event_value('-THREAD STARTED-', None)

    while True:
        cpu_percent = psutil.cpu_percent(interval=g_interval)
        procs = psutil.process_iter()
        print(f'updating with {cpu_percent}')
        window.write_event_value('-CPU UPDATE FROM THREAD-', (cpu_percent, procs))


def main():
    global g_interval

    location =  sg.user_settings_get_entry('-location-', (None, None))

    # ----------------  Create Form  ----------------
    sg.theme('Black')

    layout = [[sg.Text(font=('Helvetica', 20), text_color=sg.YELLOWS[0], key='-CPU PERCENT-')],
              [sg.Text(size=(35, 12), font=('Courier New', 12), key='-PROCESSES-')],     # size will determine how many processes shown
              [sg.Text('Update every '), sg.Spin([x+1 for x in range(10)], g_interval, key='-SPIN-'), sg.T('seconds')]]

    window = sg.Window('Top CPU Processes', layout, no_titlebar=True, keep_on_top=True,location=location, use_default_focus=False, alpha_channel=.8, grab_anywhere=True, right_click_menu=sg.MENU_RIGHT_CLICK_EDITME_VER_EXIT, enable_close_attempted_event=True, finalize=True)

    # start cpu measurement thread
    # using the PySimpleGUI call to start and manage the thread
    window.start_thread(lambda: CPU_thread(window), '-THREAD FINISHED-')
    # Unusual construct of a Try around entire event loop... something is crashing, we need to find out what...
    try:
       # ----------------  main loop  ----------------
        while True:
            # --------- Read and update window --------
            event, values = window.read()
            # sg.Print(event, values)
            # --------- Do Button Operations --------
            if event in (sg.WIN_CLOSE_ATTEMPTED_EVENT, 'Exit'):
                sg.user_settings_set_entry('-location-', window.current_location())     # save window location before exiting
                break
            if event == 'Edit Me':
                sp = sg.execute_editor(__file__)
            elif event == 'Version':
                sg.popup_scrolled(__file__, sg.get_versions(), keep_on_top=True, location=window.current_location())
            elif event == '-CPU UPDATE FROM THREAD-':                   # indicates data from the thread has arrived
                cpu_percent, procs = values[event]                      # the thread sends a tuple
                if procs:
                    # --------- Create dictionary of top % CPU processes.  Format is name:cpu_percent --------
                    top = {}
                    for proc in procs:
                        try:
                            top[proc.name()] = proc.cpu_percent()
                        except Exception as e:
                            pass    # it's OK to get an exception here because processes come and go... one may have gone...
                            # sg.Print('*** GOT Exception looping through procs ***', c='white on red', font='_ 18')
                            # sg.Print('Exception = ', e, 'procs=', procs, 'proc', proc)

                    top_sorted = sorted(top.items(), key=operator.itemgetter(1), reverse=True)  # reverse sort to get highest CPU usage on top
                    if top_sorted:
                        top_sorted.pop(0)           # remove the idle process
                    display_string =  '\n'.join([f'{cpu/10:2.2f} {proc:23}' for proc, cpu in top_sorted])
                    # --------- Display timer and proceses in window --------
                    window['-CPU PERCENT-'].update(f'CPU {cpu_percent}')
                    window['-PROCESSES-'].update(display_string)
            # get the timeout from the spinner
            g_interval = int(values['-SPIN-'])
    except Exception as e:
        sg.Print('*** GOT Exception in event loop ***', c='white on red', font='_ 18')
        sg.Print('Exception = ', e, wait=True)      # IMPORTANT to add a wait/blocking so that the print pauses execution. Otherwise program continue and exits

    window.close()

if __name__ == '__main__':
    main()



#!/usr/bin/env python

'''
Copyright 2023 PySimpleSoft, Inc. and/or its licensors. All rights reserved.

Redistribution, modification, or any other use of PySimpleGUI or any portion thereof is subject
to the terms of the PySimpleGUI License Agreement available at https://eula.pysimplegui.com.

You may not redistribute, modify or otherwise use PySimpleGUI or its contents except pursuant
to the PySimpleGUI License Agreement.
'''

import PySimpleGUI as sg
import psutil

"""
    Desktop "Rainmeter" style widget - Drive usage
    Requires: psutil
    Shows a bar graph of space used for each drive partician that psutil finds
"""

ALPHA = 0.7
THEME = 'black'
UPDATE_FREQUENCY_MILLISECONDS = 20*1000

BAR_COLORS = ('#23a0a0', '#56d856', '#be45be', '#5681d8', '#d34545', '#BE7C29')


def human_size(bytes, units=(' bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB')):
    """ Returns a human readable string reprentation of bytes"""
    return str(bytes) + ' ' + units[0] if bytes < 1024 else human_size(bytes >> 10, units[1:])

def update_window(window):
    particians = psutil.disk_partitions()
    for count, part in enumerate(particians):
        mount = part[0]
        try:
            usage = psutil.disk_usage(mount)
            window[('-NAME-', mount)].update(mount)
            window[('-PROG-', mount)].update_bar(int(usage.percent))
            window[('-%-', mount)].update(f'{usage.percent}%')
            window[('-STATS-', mount)].update(f'{human_size(usage.used)} / {human_size(usage.total)} = {human_size(usage.free)} free')
        except:
            pass

def main():
    sg.theme(THEME)

    # ----------------  Create Layout  ----------------

    layout = [[sg.Text('Drive Status', font='Any 16')]]

    # Add a row for every partician that has a bar graph and text stats
    particians = psutil.disk_partitions()
    for count, part in enumerate(particians):
        mount = part[0]
        try:
            bar_color = sg.theme_progress_bar_color()
            this_color = BAR_COLORS[count%len(BAR_COLORS)]
            usage = psutil.disk_usage(mount)
            stats_info = f'{human_size(usage.used)} / {human_size(usage.total)} = {human_size(usage.free)} free'
            layout += [[sg.Text(mount, size=(5,1), key=('-NAME-', mount)), sg.ProgressBar(100, 'h', size=(10,15), key=('-PROG-', mount), bar_color=(this_color, bar_color[1])),
                        sg.Text(f'{usage.percent}%', size=(6,1), key=('-%-', mount)), sg.T(stats_info, size=(30,1), key=('-STATS-', mount))]]
        except:
            pass
    layout += [[sg.Text('Refresh', font='Any 8', key='-REFRESH-', enable_events=True), sg.Text('❎', enable_events=True, key='Exit Text')]]

    # ----------------  Create Window  ----------------
    window = sg.Window('Drive status', layout, keep_on_top=True, grab_anywhere=True, no_titlebar=True, alpha_channel=ALPHA, use_default_focus=False, finalize=True, auto_save_location=True, right_click_menu=sg.MENU_RIGHT_CLICK_EDITME_VER_EXIT)

    update_window(window)       # sets the progress bars

    # ----------------  Event Loop  ----------------
    while True :
        event, values = window.read(timeout=UPDATE_FREQUENCY_MILLISECONDS)
        if event == sg.WIN_CLOSED or event.startswith('Exit'):
            break
        update_window(window)
        if event == 'Edit Me':
            sg.execute_editor(__file__)
        elif event == 'Version':
            sg.popup_scrolled(__file__, sg.get_versions(), location=window.current_location(), keep_on_top=True, non_blocking=True)
if __name__ == "__main__":
    main()



'''
Copyright 2023 PySimpleSoft, Inc. and/or its licensors. All rights reserved.

Redistribution, modification, or any other use of PySimpleGUI or any portion thereof is subject
to the terms of the PySimpleGUI License Agreement available at https://eula.pysimplegui.com.

You may not redistribute, modify or otherwise use PySimpleGUI or its contents except pursuant
to the PySimpleGUI License Agreement.
'''

import PySimpleGUI as sg
import psutil
import sys

"""
    Another simple Desktop Widget using PySimpleGUI
    This time a RAM indicator.  The Widget is square.  The bottom section will be shaded to 
    represent the total amount of RAM currently in use.
    The % and number of bytes in use is shown on top in text.
    Uses the theme's button color for colors.
"""

THEME = 'Dark Green 5'
GSIZE = (180, 180)
UPDATE_FREQUENCY_MILLISECONDS = 10 * 1000


def saved_alpha():
    return sg.user_settings_get_entry('-alpha-', 10)


def right_click_menu():
    alpha = saved_alpha()
    menu = [[''], [
                     'Transparency', [f'{x}{" "+sg.SYMBOL_CHECK_SMALL if x == alpha else "  "}::alpha' for x in range(1, 11)],
                     'Edit Me',
                     'Version',
                     'Exit',]
                        ]
    return menu



def human_size(bytes, units=(' bytes', 'KB', 'MB', 'GB', 'TB', 'PB', 'EB')):
    """ Returns a human readable string reprentation of bytes"""
    return str(bytes) + ' ' + units[0] if bytes < 1024 else human_size(bytes >> 10, units[1:])



def main(location):
    sg.theme(THEME)

    graph = sg.Graph(GSIZE, (0, 0), GSIZE, key='-GRAPH-', enable_events=True, tooltip='Right click for menu')
    layout = [[graph]]

    window = sg.Window('RAM Usage Widget Square', layout, location=location, no_titlebar=True, grab_anywhere=True, margins=(0, 0), element_padding=(0, 0), alpha_channel=saved_alpha()/10, finalize=True, right_click_menu=right_click_menu(), enable_close_attempted_event=True, keep_on_top=True, auto_save_location=True)


    while True:  # Event Loop
        # ----------- update the graphics and text in the window ------------
        ram = psutil.virtual_memory()
        rect_height = int(GSIZE[1] * float(ram.percent) / 100)
        rect_id = graph.draw_rectangle((0, rect_height), (GSIZE[0], 0), fill_color=sg.theme_button_color()[1], line_width=0)
        text_id1 = graph.draw_text(f'{int(ram.percent)}%', (GSIZE[0] // 2, GSIZE[1] // 2), font='Any 40', text_location=sg.TEXT_LOCATION_CENTER,
                                   color=sg.theme_button_color()[0])
        text_id2 = graph.draw_text(f'{human_size(ram.used)} used', (GSIZE[0] // 2, GSIZE[1] // 4), font='Any 20', text_location=sg.TEXT_LOCATION_CENTER, color=sg.theme_button_color()[0])

        event, values = window.read(timeout=UPDATE_FREQUENCY_MILLISECONDS)
        if event in (sg.WIN_CLOSED, 'Exit', sg.WIN_CLOSE_ATTEMPTED_EVENT):
            if event != sg.WIN_CLOSED:
                sg.user_settings_set_entry('-location-', window.current_location())  # The line of code to save the position before exiting
            break

        if event.endswith('alpha') and not sg.SYMBOL_CHECK_SMALL in event:     # if a new alpha is chosen
            alpha = int(event.split('::')[0])
            window.set_alpha(alpha/10)
            sg.user_settings_set_entry('-alpha-', alpha)
            window['-GRAPH-'].set_right_click_menu(right_click_menu())

        if event == 'Edit Me':
            sg.execute_editor(__file__)
        elif event == 'Version':
            sg.popup_scrolled(__file__, sg.get_versions(), location=window.current_location(), keep_on_top=True, non_blocking=True)

        graph.delete_figure(rect_id)
        graph.delete_figure(text_id1)
        graph.delete_figure(text_id2)
    window.close()

if __name__ == '__main__':

    location = sg.user_settings_get_entry('-location-', (None, None))
    main(location)



'''
Copyright 2023 PySimpleSoft, Inc. and/or its licensors. All rights reserved.

Redistribution, modification, or any other use of PySimpleGUI or any portion thereof is subject
to the terms of the PySimpleGUI License Agreement available at https://eula.pysimplegui.com.

You may not redistribute, modify or otherwise use PySimpleGUI or its contents except pursuant
to the PySimpleGUI License Agreement.
'''

import PySimpleGUI as sg
import datetime
import base64
from urllib import request
import json
import sys
import webbrowser

"""
    A Current Weather Widget

    Adapted from the weather widget originally created and published by Israel Dryer that you'll find here:
    https://github.com/israel-dryer/Weather-App

    BIG THANKS goes out for creating a good starting point for other widgets to be build from.

    A true "Template" is being developed that is a little more abstracted to make creating your own
    widgets easy.  Things like the settings window is being standardized, the settings file format too.

    You will need a key (APPID) from OpenWeathermap.org in order to run this widget. It's free, it's easy:
    https://home.openweathermap.org/

    Your initial location is determined using your IP address and will be used if no settings file is found

    This widget is an early version of a PSG Widget so it may not share the same names / constructs as the templates. 

"""

SETTINGS_PATH = '.'

API_KEY = ''  # Set using the "Settings" window and saved in your config file

sg.theme('Light Green 6')
ALPHA = 0.8

BG_COLOR = sg.theme_text_color()
TXT_COLOR = sg.theme_background_color()

APP_DATA = {
    'City': 'New York',
    'Country': 'US',
    'Postal': 10001,
    'Description': 'clear skys',
    'Temp': 101.0,
    'Feels Like': 72.0,
    'Wind': 0.0,
    'Humidity': 0,
    'Precip 1hr': 0.0,
    'Pressure': 0,
    'Updated': 'Not yet updated',
    'Icon': None,
    'Units': 'Imperial'
}


def load_settings():
    global API_KEY
    settings = sg.UserSettings(path=SETTINGS_PATH)
    API_KEY = settings['-api key-']
    if not API_KEY:
        sg.popup_quick_message('No valid API key found... opening setup window...', keep_on_top=True, background_color='red', text_color='white',
                               auto_close_duration=3, non_blocking=False)
        change_settings(settings)
    return settings


def change_settings(settings, window_location=(None, None)):
    global APP_DATA, API_KEY

    try:
        nearest_postal = json.loads(request.urlopen('http://ipapi.co/json').read())['postal']
    except Exception as e:
        print('Error getting nearest postal', e)
        nearest_postal = ''

    layout = [[sg.T('Enter Zipcode or City for your location')],
              [sg.I(settings.get('-location-', nearest_postal), size=(15, 1), key='-LOCATION-')],
              [sg.I(settings.get('-api key-', ''), size=(32, 1), key='-API KEY-')],
              [sg.B('Ok', border_width=0, bind_return_key=True),  sg.B('Register For a Key', border_width=0, k='-REGISTER-'), sg.B('Cancel', border_width=0)], ]

    window = sg.Window('Settings', layout, location=window_location, no_titlebar=True, keep_on_top=True, border_depth=0)
    event, values = window.read()
    window.close()

    if event == '-REGISTER-':
        sg.popup('Launching browser so you can signup for the "Current Weather" service from OpenWeatherMap.org to get a Free API Key', 'Click OK and your browser will open', r'Visit https://home.openweathermap.org/ for more information', location=window_location)
        # Register to get a free key
        webbrowser.open(r'https://home.openweathermap.org/users/sign_up')


    if event == 'Ok':
        user_location = settings['-location-'] = values['-LOCATION-']
        API_KEY = settings['-api key-'] = values['-API KEY-']
    else:
        API_KEY = settings['-api key-']
        user_location = settings['-location-']

    if user_location is not None:
        if user_location.isnumeric() and len(user_location) == 5 and user_location is not None:
            APP_DATA['Postal'] = user_location
            APP_DATA['City'] = ''
        else:
            APP_DATA['City'] = user_location
            APP_DATA['Postal'] = ''

    return settings


def update_weather():
    if APP_DATA['City']:
        request_weather_data(create_endpoint(2))
    elif APP_DATA['Postal']:
        request_weather_data(create_endpoint(1))


def create_endpoint(endpoint_type=0):
    """ Create the api request endpoint
    {0: default, 1: zipcode, 2: city_name}"""
    if endpoint_type == 1:
        try:
            endpoint = f"http://api.openweathermap.org/data/2.5/weather?zip={APP_DATA['Postal']},us&appid={API_KEY}&units={APP_DATA['Units']}"
            return endpoint
        except ConnectionError:
            return
    elif endpoint_type == 2:
        try:
            endpoint = f"http://api.openweathermap.org/data/2.5/weather?q={APP_DATA['City'].replace(' ', '%20')},us&APPID={API_KEY}&units={APP_DATA['Units']}"
            return endpoint
        except ConnectionError:
            return
    else:
        return


def request_weather_data(endpoint):
    """ Send request for updated weather data """
    global APP_DATA

    if endpoint is None:
        sg.popup_error('Could not connect to api.  endpoint is None', keep_on_top=True)
        return
    else:
        try:
            response = request.urlopen(endpoint)
        except request.HTTPError:
            sg.popup_error('ERROR Obtaining Weather Data',
                           'Is your API Key set correctly?',
                           API_KEY, keep_on_top=True)
            return

    if response.reason == 'OK':
        weather = json.loads(response.read())
        APP_DATA['City'] = weather['name'].title()
        APP_DATA['Description'] = weather['weather'][0]['description']
        APP_DATA['Temp'] = "{:,.0f}°F".format(weather['main']['temp'])
        APP_DATA['Humidity'] = "{:,d}%".format(weather['main']['humidity'])
        APP_DATA['Pressure'] = "{:,d} hPa".format(weather['main']['pressure'])
        APP_DATA['Feels Like'] = "{:,.0f}°F".format(weather['main']['feels_like'])
        APP_DATA['Wind'] = "{:,.1f} m/h".format(weather['wind']['speed'])
        APP_DATA['Precip 1hr'] = None if not weather.get('rain') else "{:2} mm".format(weather['rain']['1h'])
        APP_DATA['Updated'] = 'Updated: ' + datetime.datetime.now().strftime("%B %d %I:%M:%S %p")
        APP_DATA['Lon'] = weather['coord']['lon']
        APP_DATA['Lat'] = weather['coord']['lat']

        icon_url = "http://openweathermap.org/img/wn/{}@2x.png".format(weather['weather'][0]['icon'])
        APP_DATA['Icon'] = base64.b64encode(request.urlopen(icon_url).read())


def metric_row(metric):
    """ Return a pair of labels for each metric """
    return [sg.Text(metric, font=('Arial', 10), pad=(15, 0), size=(9, 1)),
            sg.Text(APP_DATA[metric], font=('Arial', 10, 'bold'), pad=(0, 0), size=(9, 1), key=metric)]


def create_window(win_location):
    """ Create the application window """
    col1 = sg.Column(
        [[sg.Text(APP_DATA['City'], font=('Arial Rounded MT Bold', 18), pad=((10, 0), (50, 0)), size=(18, 1), background_color=BG_COLOR, text_color=TXT_COLOR,
                  key='City')],
         [sg.Text(APP_DATA['Description'], font=('Arial', 12), pad=(10, 0), background_color=BG_COLOR, text_color=TXT_COLOR, key='Description')]],
        background_color=BG_COLOR, key='COL1')

    col2 = sg.Column(
        [[sg.Text('×', font=('Arial Black', 16), pad=(0, 0), justification='right', background_color=BG_COLOR, text_color=TXT_COLOR, enable_events=True,
                  key='-QUIT-')],
         [sg.Image(data=APP_DATA['Icon'], pad=((5, 10), (0, 0)), size=(100, 100), background_color=BG_COLOR, key='Icon')]],
        element_justification='center', background_color=BG_COLOR, key='COL2')

    col3 = sg.Column(
        [[sg.Text(APP_DATA['Updated'], font=('Arial', 8), background_color=BG_COLOR, text_color=TXT_COLOR, key='Updated')]],
        pad=(10, 5), element_justification='left', background_color=BG_COLOR, key='COL3')

    col4 = sg.Column(
        [[sg.Text('Settings', font=('Arial', 8, 'italic'), background_color=BG_COLOR, text_color=TXT_COLOR, enable_events=True, key='-CHANGE-'),
          sg.Text('Refresh', font=('Arial', 8, 'italic'), background_color=BG_COLOR, text_color=TXT_COLOR, enable_events=True, key='-REFRESH-')]],
        pad=(10, 5), element_justification='right', background_color=BG_COLOR, key='COL4')

    top_col = sg.Column([[col1, col2]], pad=(0, 0), background_color=BG_COLOR, key='TopCOL')

    bot_col = sg.Column([[col3, col4]], pad=(0, 0), background_color=BG_COLOR, key='BotCOL')

    lf_col = sg.Column(
        [[sg.Text(APP_DATA['Temp'], font=('Haettenschweiler', 90), pad=((10, 0), (0, 0)), justification='center', key='Temp')]],
        pad=(10, 0), element_justification='center', key='LfCOL')

    rt_col = sg.Column(
        [metric_row('Feels Like'), metric_row('Wind'), metric_row('Humidity'), metric_row('Precip 1hr'), metric_row('Pressure')],
        pad=((15, 0), (25, 5)), key='RtCOL')

    layout = [[top_col],
              [lf_col, rt_col],
              [bot_col]]

    window = sg.Window(layout=layout, title='Weather Widget', margins=(0, 0), finalize=True, location=win_location,
                       element_justification='center', keep_on_top=True, no_titlebar=True, grab_anywhere=True, alpha_channel=ALPHA, right_click_menu=sg.MENU_RIGHT_CLICK_EDITME_VER_EXIT, auto_save_location=True)

    for col in ['COL1', 'COL2', 'TopCOL', 'BotCOL', '-QUIT-']:
        window[col].expand(expand_y=True, expand_x=True)

    for col in ['COL3', 'COL4', 'LfCOL', 'RtCOL']:
        window[col].expand(expand_x=True)

    window['-CHANGE-'].set_cursor('hand2')
    window['-QUIT-'].set_cursor('hand2')
    window['-REFRESH-'].set_cursor('hand2')

    return window


def update_metrics(window):
    """ Adjust the GUI to reflect the current weather metrics """
    metrics = ['City', 'Temp', 'Feels Like', 'Wind', 'Humidity', 'Precip 1hr',
               'Description', 'Icon', 'Pressure', 'Updated']
    for metric in metrics:
        if metric == 'Icon':
            window[metric].update(data=APP_DATA[metric])
        else:
            window[metric].update(APP_DATA[metric])


def main(refresh_rate, win_location):
    """ The main program routine """
    refresh_in_milliseconds = refresh_rate * 60 * 1000

    # Load settings from config file. If none found will create one
    settings = load_settings()
    location = settings['-location-']
    if location is not None:
        if location.isnumeric() and len(location) == 5 and location is not None:
            APP_DATA['Postal'] = location
            APP_DATA['City'] = ''
        else:
            APP_DATA['City'] = location
            APP_DATA['Postal'] = ''
        update_weather()
    else:
        sg.popup_error('Having trouble with location.  Your location: ', location)
        exit()

    window = create_window(win_location)

    while True:  # Event Loop
        event, values = window.read(timeout=refresh_in_milliseconds)
        if event in (None, '-QUIT-', 'Exit'):
            break
        if event == '-CHANGE-':
            x, y = window.current_location()
            settings = change_settings(settings, (x + 405, y))
        elif event == '-REFRESH-':
            sg.popup_quick_message('Refreshing...', keep_on_top=True, background_color='red', text_color='white',
                                   auto_close_duration=3, non_blocking=False)
        elif event == 'Edit Me':
            sg.execute_editor(__file__)
        elif event == 'Version':
            sg.popup_scrolled(__file__, sg.get_versions(), location=window.current_location(), keep_on_top=True, non_blocking=True)
        update_weather()
        update_metrics(window)
    window.close()


if __name__ == '__main__':
    if len(sys.argv) > 1:
        location = sys.argv[1].split(',')
        location = (int(location[0]), int(location[1]))
    else:
        location = (None, None)
    main(refresh_rate=1, win_location=location)



'''
Copyright 2023 PySimpleSoft, Inc. and/or its licensors. All rights reserved.

Redistribution, modification, or any other use of PySimpleGUI or any portion thereof is subject
to the terms of the PySimpleGUI License Agreement available at https://eula.pysimplegui.com.

You may not redistribute, modify or otherwise use PySimpleGUI or its contents except pursuant
to the PySimpleGUI License Agreement.
'''

import PySimpleGUI as sg
import psutil
import sys

"""
    Another simple Desktop Widget using PySimpleGUI
    Right click the window to exit
    This time a CPU Usage indicator.  The Widget is square.  
    The bottom section will be shaded to 
    represent the total amount CPU currently in use.
    Uses the theme's button color for colors.

"""

THEME = 'Dark purple 6'
GSIZE = (180, 180)
UPDATE_FREQUENCY_MILLISECONDS = 2 * 1000

def saved_alpha():
    return sg.user_settings_get_entry('-alpha-', 10)


def right_click_menu():
    alpha = saved_alpha()
    menu = [[''], [
                     'Transparency', [f'{x}{" "+sg.SYMBOL_CHECK_SMALL if x == alpha else "  "}::alpha' for x in range(1, 11)],
                     'Edit Me',
                     'Version',
                     'Exit',]
                        ]
    return menu


def main(location):
    graph = sg.Graph(GSIZE, (0, 0), GSIZE, key='-GRAPH-', tooltip='Right click for menu')

    layout = [[graph]]

    window = sg.Window('CPU Usage Widget Square', layout, location=location, no_titlebar=True, grab_anywhere=True, margins=(0, 0), element_padding=(0, 0), alpha_channel=saved_alpha()/10, finalize=True, right_click_menu=right_click_menu(), enable_close_attempted_event=True, auto_save_location=True)

    text_id2 = graph.draw_text(f'CPU', (GSIZE[0] // 2, GSIZE[1] // 4), font='Any 20', text_location=sg.TEXT_LOCATION_CENTER, color=sg.theme_button_color()[0])


    while True:  # Event Loop
        # ----------- update the graphics and text in the window ------------
        cpu_percent = psutil.cpu_percent(interval=1)
        # Draw the filled rectangle
        rect_height = int(GSIZE[1] * float(cpu_percent) / 100)
        rect_id = graph.draw_rectangle((0, rect_height), (GSIZE[0], 0), fill_color=sg.theme_button_color()[1], line_width=0)
        # Draw the % used text and the close "X" on bottom
        text_id1 = graph.draw_text(f'{int(cpu_percent)}%', (GSIZE[0] // 2, GSIZE[1] // 2), font='Any 40', text_location=sg.TEXT_LOCATION_CENTER, color=sg.theme_button_color()[0])
        # put the bar behind everything else
        graph.send_figure_to_back(rect_id)

        # update the window, wait for a while, then check for exit
        event, values = window.read(timeout=UPDATE_FREQUENCY_MILLISECONDS)
        if event in (sg.WIN_CLOSE_ATTEMPTED_EVENT, 'Exit'):
            sg.user_settings_set_entry('-location-', window.current_location())  # The line of code to save the position before exiting
            break
        if event.endswith('alpha') and not sg.SYMBOL_CHECK_SMALL in event:     # if a new alpha is chosen
            alpha = int(event.split('::')[0])
            window.set_alpha(alpha/10)
            sg.user_settings_set_entry('-alpha-', alpha)
            window['-GRAPH-'].set_right_click_menu(right_click_menu())

        if event == 'Edit Me':
            sg.execute_editor(__file__)
        elif event == 'Version':
            sg.popup_scrolled(__file__, sg.get_versions(), location=window.current_location(), keep_on_top=True, non_blocking=True)
        # erase figures so they can be redrawn
        graph.delete_figure(rect_id)
        graph.delete_figure(text_id1)
    window.close()


if __name__ == '__main__':
    sg.theme(THEME)

    location = sg.user_settings_get_entry('-location-', (None, None))
    main(location)





import setuptools

def readme():
    try:
        with open("README.md") as f:
            return f.read()
    except IOError:
        return ""


setuptools.setup(
name="psggadgets",
version="5.0.0",
author="PySimpleSoft Inc.",
install_requires=["PySimpleGUI>=5","psutil"],
description="Desktop Gadgets Using PySimpleGUI",
long_description=readme(),
long_description_content_type="text/markdown",
license="Free To Use But Restricted",
keywords="desktop widget rainmeter gadgets windows GUI PySimpleGUI",
url="https://github.com/PySimpleGUI/psggadgets",
packages=setuptools.find_packages(),
python_requires=">=3.6",
classifiers=[
    "Intended Audience :: Developers",
    "License :: Free To Use But Restricted",
    "Operating System :: OS Independent",
    "Framework :: PySimpleGUI",
    "Framework :: PySimpleGUI :: 5",
    "Programming Language :: Python :: 3",
    "Programming Language :: Python :: 3.6",
    "Programming Language :: Python :: 3.7",
    "Programming Language :: Python :: 3.8",
    "Programming Language :: Python :: 3.9",
    "Programming Language :: Python :: 3.10",
    "Programming Language :: Python :: 3.11",
    "Programming Language :: Python :: 3.12",
    "Programming Language :: Python :: 3.13",
    "Topic :: Multimedia :: Graphics",
],
package_data={"":
["*","*.*"]
        },
entry_points={"gui_scripts": [
            "psggadgets=psggadgets.psggadgets:main"
],},
)
